/*
 * IRLog.c
 *
 *  Created on: 07.07.2011
 *      Author: jmm
 */

#include <avr/interrupt.h>
#include <stdbool.h>
#include "uartlib.h"
#include "timelib.h"
#include "defines.h"

volatile timeT buffer[BUFFER_SIZE];
volatile uint16_t bufferWrite = 0;
volatile uint16_t bufferRead = 0;

inline void buffer_put(timeT value) {
	// Ablegen
	*(buffer + bufferWrite) = value;
	// increment
	bufferWrite = (bufferWrite + 1) % BUFFER_SIZE;
}

inline timeP buffer_get() {
	// Pointer auf aktuelles Element sichern
	timeP ret = (timeP) buffer + bufferRead;
	// increment
	bufferRead = (bufferRead + 1) % BUFFER_SIZE;
	return ret;
}

inline bool buffer_empty() {
	return (bufferRead == bufferWrite);
}

inline bool buffer_full() {
	return ((bufferWrite + 1) % BUFFER_SIZE == bufferRead);
}

ISR(INT1_vect) {
	if(!buffer_full())
		buffer_put(systemTime);
}

int main() {
	// Interrupt bei Flanke an INT1 (auch ein PCINT möglich!)
	cbi(DDRD, DDD3);
	MCUCR &= ~(3<<ISC10);
	sbi(MCUCR, ISC10);
	sbi(GICR, INT1);

	// Initialisierungen
	systemtime_init();
	uart_init();
	uart_transmit8(255);

	sei();

	while(1) {
		while(!buffer_empty()) {
			uart_transmit32P(buffer_get());
		}
	}
}
