/*
 * uart.c
 *
 *  Created on: 08.07.2011
 *      Author: jmm
 */

#include <avr/interrupt.h>
#include "uartlib.h"

#define UBRR_VAL 	((F_CPU + BAUD * 8UL) / (BAUD * 16UL) - 1UL)

inline void uart_init() {
	UBRRH = (uint8_t) (UBRR_VAL >> 8);
	UBRRL = (uint8_t) UBRR_VAL;
	UCSRB = (1 << TXEN);
	UCSRC = (1 << URSEL)
			| (3 << UCSZ0);
}

inline void uart_transmit8(uint8_t data) {
	while ( !( UCSRA & (1<<UDRE)) );
	UDR = data;
}

inline void uart_transmit8P(uint8_t* data) {
	uart_transmit8(*data);
}

inline void uart_transmit16(uint16_t data) {
	uart_transmit8(data >> 8);
	uart_transmit8(data);
}

inline void uart_transmit16P(uint16_t* data) {
	uint8_t* dataX = (uint8_t*) data;
	uart_transmit8(*(dataX + 1));
	uart_transmit8(*(dataX + 0));
}

inline void uart_transmit32(uint32_t data) {
	uart_transmit8(data >> 24);
	uart_transmit8(data >> 16);
	uart_transmit8(data >> 8);
	uart_transmit8(data);
}

inline void uart_transmit32P(uint32_t* data) {
	uint8_t* dataX = (uint8_t*) data;
	uart_transmit8(*(dataX + 3));
	uart_transmit8(*(dataX + 2));
	uart_transmit8(*(dataX + 1));
	uart_transmit8(*(dataX + 0));
}

inline void uart_transmit64(uint64_t data) {
	uart_transmit8(data >> 56);
	uart_transmit8(data >> 48);
	uart_transmit8(data >> 40);
	uart_transmit8(data >> 32);
	uart_transmit8(data >> 24);
	uart_transmit8(data >> 16);
	uart_transmit8(data >> 8);
	uart_transmit8(data);
}

inline void uart_transmit64P(uint64_t* data) {
	uint8_t* dataX = (uint8_t*) data;
	uart_transmit8(*(dataX + 7));
	uart_transmit8(*(dataX + 6));
	uart_transmit8(*(dataX + 5));
	uart_transmit8(*(dataX + 4));
	uart_transmit8(*(dataX + 3));
	uart_transmit8(*(dataX + 2));
	uart_transmit8(*(dataX + 1));
	uart_transmit8(*(dataX + 0));
}
